﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    /// <summary>
    /// 通用的扩展方法集合
    /// </summary>
    internal static class Extension
    {
        /// <summary>
        /// 获取枚举值的描述特性
        /// </summary>
        /// <param name="value">要获取描述的枚举值</param>
        /// <returns>返回枚举值的描述；如果没有描述特性，则返回枚举值的名称</returns>
        /// <example>
        /// <code>
        /// Status status = Status.Pending;
        /// string description = status.GetDescription(); // 返回 "待处理"
        /// </code>
        /// </example>
        public static string GetDescription(this Enum value)
        {
            // 通过反射获取枚举值对应的字段信息
            FieldInfo field = value.GetType().GetField(value.ToString());

            if (field != null)
            {
                // 从字段中获取DescriptionAttribute特性
                DescriptionAttribute attribute = Attribute.GetCustomAttribute(field,
                    typeof(DescriptionAttribute)) as DescriptionAttribute;

                if (attribute != null && !string.IsNullOrEmpty(attribute.Description))
                {
                    // 如果找到了描述特性，则返回其Description属性值
                    return attribute.Description;
                }
            }

            // 如果没有找到描述特性或字段信息为空，则返回枚举值的字符串表示
            return value.ToString();
        }

        /// <summary>
        /// 获取指定枚举类型的所有值和描述，并返回为键值对列表
        /// </summary>
        /// <typeparam name="T">枚举类型</typeparam>
        /// <returns>包含枚举值和描述的键值对列表</returns>
        public static List<KeyValuePair<string, T>> GetAllValuesAndDescriptions<T>() where T : Enum
        {
            // 创建结果列表
            List<KeyValuePair<string, T>> result = new List<KeyValuePair<string, T>>();

            // 获取枚举类型的所有值
            Array enumValues = Enum.GetValues(typeof(T));

            // 遍历所有枚举值
            foreach (T value in enumValues)
            {
                // 获取当前枚举值的描述
                string description = GetDescription((Enum)(object)value);

                // 添加到结果列表
                result.Add(new KeyValuePair<string, T>(description, value));
            }

            return result;
        }
    }
}
